function u = ncLogTVL2_IRL1 (f, g, lambda, beta, maxiter)
   
  % Algorithm parameters
  maxIIter = 50;
  eTOL_out = 1e-06;
  L = sqrt(8);
  theta = 1;
  tau = 1/L*0.1;
  sigma = 1/L/0.1;
  
  % initialization
  [M,N] = size(f);
  u = f;
  u_ = u;
  p = zeros(M,N,2);
  
  % compute intial energy
  e0 = energy_LogTVL2(u, f, lambda, beta);
  e_old = e0;

  it_out = 0;
  while(true) 
    % update weights
    u_x = dxp(u_);
    u_y = dyp(u_);
    weight = beta./(1+beta*sqrt(u_x.^2+u_y.^2));
  
    for it_in=1:maxIIter
      %% inner (convex) probelm
      it_out = it_out + 1;

      % update dual
      u_x = dxp(u_);
      u_y = dyp(u_);
      p(:,:,1) = p(:,:,1) + sigma.*u_x;
      p(:,:,2) = p(:,:,2) + sigma.*u_y;
      reproj = max(1.0, sqrt(p(:,:,1).^2+p(:,:,2).^2)./weight);
      p(:,:,1) = p(:,:,1)./reproj;
      p(:,:,2) = p(:,:,2)./reproj;   
             
      % remember u
      u_ = u;
      
      % acceleration
      theta = 1/sqrt(1+0.7*lambda*tau);
      tau = tau*theta;
      sigma = sigma/theta;
      
      % update primal
      div = dxm(p(:,:,1)) + dym(p(:,:,2));     
      u = (u + tau*(div+lambda*f))./(1+tau*lambda);     
            
      % extra-gradient step 
      u_ = u + theta*(u-u_);
      
      % update energy
      e = energy_LogTVL2(u, f, lambda, beta);

      % show
      if (mod(it_in,10)==0)
          fprintf('it_out: %g, it_in: %g, energy: %f\n', it_out, it_in, e);
          sfigure(1); imshow([g f u], [0 1]); drawnow;
      
          % breaking condition for inner iterations
          if (e<e_old); break; end
      end
      
    end
      
    % breaking condition for outer itations
    if (abs(e_old-e)/e0 < eTOL_out || it_out > maxiter); break; end

    % update outer energy
    e_old = e;
  end

end
function energy = energy_LogTVL2(u, f, lambda, beta)
  % E = sum_pixel { 0.5*lambda*(u-f).^2 + log(1+beta*sqrt(u_x.^2+u_y.^2)) }
  u_x = dxp(u);
  u_y = dyp(u);
  energy = sum(sum( 0.5*lambda*(u-f).^2+log(1+beta*sqrt(u_x.^2+u_y.^2)) ));
end

