function u = ncpnormTGVL2_IRL1 (f, g, lambda, pnorm, alpha1, alpha2, maxiter)

  % Algorithm parameters
  maxIIter = 400;
  eTOL_out = 1e-06;
  EPS = 1e-6;
  L = sqrt(12);
  theta = 1;
  tau = 1/L*0.1;
  sigma = 1/L/0.1;
  
  % initialization
  [M,N] = size(f);
  u = f;
  u_ = u;
  v = zeros(M,N,2);
  v_ = v;
  p = zeros(M,N,2);
  q = zeros(M,N,4);

  % compute intial energy
  e0 = energy_pnormTGVL2(u, v, f, lambda, pnorm, alpha1, alpha2, EPS);
  e_old = e0;
      
  it_out = 0;
  while(true) 
    % update weights
    u_x = dxp(u_);
    u_y = dyp(u_);
    weight_u = (alpha1*pnorm)./power(sqrt(u_x.^2+u_y.^2)+EPS,1-pnorm);
    v1_x = dxp(v_(:,:,1));
    v1_y = dyp(v_(:,:,1));
    v2_x = dxp(v_(:,:,2));
    v2_y = dyp(v_(:,:,2));
    weight_v = (alpha2*pnorm)./power(sqrt(v1_x.^2+v1_y.^2+v2_x.^2+v2_y.^2)+EPS,1-pnorm);
    
    for it_in=1:maxIIter
      %% inner (convex) probelm
      it_out = it_out + 1;

      % update dual
      u_x = dxp(u_);
      u_y = dyp(u_);
      v1_x = dxp(v_(:,:,1));
      v1_y = dyp(v_(:,:,1));
      v2_x = dxp(v_(:,:,2));
      v2_y = dyp(v_(:,:,2));
      p(:,:,1) = p(:,:,1) + sigma.*(u_x-v_(:,:,1));
      p(:,:,2) = p(:,:,2) + sigma.*(u_y-v_(:,:,2));
      q(:,:,1) = q(:,:,1) + sigma.*(v1_x);
      q(:,:,2) = q(:,:,2) + sigma.*(v1_y);
      q(:,:,3) = q(:,:,3) + sigma.*(v2_x);
      q(:,:,4) = q(:,:,4) + sigma.*(v2_y);
      reproj = max(1.0, sqrt(p(:,:,1).^2+p(:,:,2).^2)./weight_u);
      p(:,:,1) = p(:,:,1)./reproj;
      p(:,:,2) = p(:,:,2)./reproj;   
      reproj = max(1.0, sqrt(q(:,:,1).^2+q(:,:,2).^2+q(:,:,3).^2+q(:,:,4).^2)./weight_v);
      q(:,:,1) = q(:,:,1)./reproj;
      q(:,:,2) = q(:,:,2)./reproj;
      q(:,:,3) = q(:,:,3)./reproj;
      q(:,:,4) = q(:,:,4)./reproj;
             
      % remember u
      u_ = u;
      v_ = v;
      
      % update primal
      div = cat(3, dxm(q(:,:,1)) + dym(q(:,:,2)), dxm(q(:,:,3)) + dym(q(:,:,4)) );     
      v = v + tau*(div+p);
      div = dxm(p(:,:,1)) + dym(p(:,:,2));     
      u = (u + tau*(div+lambda*f))./(1+tau*lambda);     
            
      % extra-gradient step 
      u_ = u + theta*(u-u_);
      v_ = v + theta*(v-v_);
     
      % update energy      
      e = energy_pnormTGVL2(u, v, f, lambda, pnorm, alpha1, alpha2, EPS);
      
      % show
      if (mod(it_in,10)==0)
          fprintf('it_out: %g, it_in: %g, energy: %f\n', it_out, it_in, e);
          sfigure(1); imshow([g f u], [0 1]); drawnow;

          if (e<e_old); break; end
      end
      
    end    
    
    % breaking condition for outer itations
    if (abs(e_old-e)/e0 < eTOL_out || it_out > maxiter); break; end

    % update outer energy
    e_old = e;
  end

end
function energy = energy_pnormTGVL2(u, v, f, lambda, pnorm, alpha1, alpha2, EPS)
% E = sum_pixel { 0.5*lambda*(u-f).^2+power(sqrt(u_x.^2+u_y.^2)+EPS, pnorm)
%                                    +power(sqrt(v1_x.^2+v1_y.^2+v2_x.^2+v2_y.^2)+EPS, pnorm) }
  u_x = dxp(u);
  u_y = dyp(u);
  v1_x = dxp(v(:,:,1));
  v1_y = dyp(v(:,:,1));
  v2_x = dxp(v(:,:,2));
  v2_y = dyp(v(:,:,2));
  
  energy = sum(sum( 0.5*lambda*(u-f).^2 ...
                +alpha1*power(sqrt(u_x.^2+u_y.^2)+EPS, 1-pnorm) ...
                +alpha2*power(sqrt(v1_x.^2+v1_y.^2+v2_x.^2+v2_y.^2)+EPS, 1-pnorm)));
end

