function u = ncLogTVL2_IRL1 (f, g, lambda, maxiter)
   
  % Algorithm parameters
  L = sqrt(8);
  theta = 1;
  tau = 1/L*0.1;
  sigma = 1/L/0.1;
  
  % initialization
  [M,N] = size(f);
  u = f;
  u_ = u;
  p = zeros(M,N,2);
  
  iter = 0;
  while(true) 
    iter = iter + 1;

    % update dual
    u_x = dxp(u_);
    u_y = dyp(u_);
    p(:,:,1) = p(:,:,1) + sigma.*u_x;
    p(:,:,2) = p(:,:,2) + sigma.*u_y;
    reproj = max(1.0, sqrt(p(:,:,1).^2+p(:,:,2).^2));
    p(:,:,1) = p(:,:,1)./reproj;
    p(:,:,2) = p(:,:,2)./reproj;   
           
    % remember u
    u_ = u;
    
    % acceleration
    theta = 1/sqrt(1+0.7*lambda*tau);
    tau = tau*theta;
    sigma = sigma/theta;
    
    % update primal
    div = dxm(p(:,:,1)) + dym(p(:,:,2));     
    u = (u + tau*(div+lambda*f))./(1+tau*lambda);     
          
    % extra-gradient step 
    u_ = u + theta*(u-u_);
    
    % show
    if (mod(iter,10)==0)
        fprintf('iter: %g\n', iter);
        sfigure(1); imshow([g f u], [0 1]); drawnow;
    end

    if (iter > maxiter); break; end
  end

end

