clear; close all

addpath('common');
addpath('data');

filename = 'watercastle';
total_filename = strcat(filename, '.png');
randn('seed',2305978);
sigma = 25;
fc = im2double(imread(total_filename));
fc = imresize(fc, 0.3);
f = fc + randn(size(fc))*sigma/255;

if 1
  psnr = compute_psnr(f,fc);
  disp(sprintf('PSNR for the noisy image: %f db\n', psnr));
  imwrite(f, [filename '_noisy.png']);
end


% >>> test TV L2 (ROF) denosing <<<   
if 1
  % PARAMETER
  maxiter = 5000;
  lambda = 10;
  
  % solve primal dual
  tic;
      u = primal_dual_TVL2(f, fc, lambda, maxiter);
  toc
  
  % write output
  psnr = compute_psnr(u,fc);
  disp(sprintf('PSNR TV: %f db\n', psnr));
  imwrite(u, [filename 'TVL2.png']);
end

% >>> test nonconvex LogTV L2 denosing <<<   
if 1
  % PARAMETER
  maxiter = 5000;
  lambda = 20;
  beta = 5.0;
 
  % solve primal dual
  tic;
      u = ncLogTVL2_IRL1 (f, fc, lambda, beta, maxiter);
  toc
  
  % write output
  psnr = compute_psnr(u,fc);
  disp(sprintf('PSNR ncLogTV: %f db\n', psnr));
  imwrite(u, [filename 'ncLogTV.png']);
end

% >>> test nonconvex pnormTV L2 denosing <<<   
if 1
  % PARAMETER
  maxiter = 5000;
  lambda = 20;
  pnorm = 0.7;
 
  % solve primal dual
  tic;
      u = ncpnormTVL2_IRL1 (f, fc, lambda, pnorm, maxiter);
  toc
  
  % write output
  psnr = compute_psnr(u,fc);
  disp(sprintf('PSNR ncpnormTV: %f db\n', psnr));
  imwrite(u, [filename 'ncpnormTV.png']);
end

% >>> test TGV L2 denosing <<<   
if 1
  % PARAMETER
  maxiter = 5000;
  lambda = 15;
  alpha1 = 0.5;
  alpha2 = 1.0;
 
  % solve primal dual
  tic;
      u = primal_dual_TGVL2 (f, fc, lambda, alpha1, alpha2, maxiter);
  toc
  
  % write output
  psnr = compute_psnr(u,fc);
  disp(sprintf('PSNR TGV: %f db\n', psnr));
  imwrite(u, [filename 'TGV.png']);
end

% >>> test nonconvex LogTGV L2 denosing <<<   
if 1
  % PARAMETER
  maxiter = 5000;
  lambda = 20;
  alpha1 = 0.5;
  alpha2 = 1.0;
  beta = 5.0;
 
  % solve primal dual
  tic;
      u = ncLogTGVL2_IRL1 (f, fc, lambda, beta, alpha1, alpha2, maxiter);
  toc
  
  % write output
  psnr = compute_psnr(u,fc);
  disp(sprintf('PSNR ncLogTGV: %f db\n', psnr));
  imwrite(u, [filename 'ncLogTGV.png']);
end

% >>> test nonconvex pnormTGV L2 denosing <<<   
if 1
  % PARAMETER
  maxiter = 5000;
  lambda = 10;
  alpha1 = 0.5;
  alpha2 = 1.0;
  pnorm = 0.7;
 
  % solve primal dual
  tic;
      u = ncpnormTGVL2_IRL1 (f, fc, lambda, pnorm, alpha1, alpha2, maxiter);
  toc
  
  % write output
  psnr = compute_psnr(u,fc);
  disp(sprintf('PSNR ncpnormTGV: %f db\n', psnr));
  imwrite(u, [filename 'ncpnormTGV.png']);
end

